@@ -1,11 +1,12 @@
 import re
+import warnings
 from datetime import date, datetime
 from typing import Union
 
 import jsonutils.base as base
 import jsonutils.config as config
-from jsonutils.exceptions import JSONQueryException
 import jsonutils.functions.parsers as parsers
+from jsonutils.exceptions import JSONQueryException
 
 
 class SingleQuery:
@@ -186,6 +187,47 @@ class ParentList(list):
     pass
 
 
+class ValuesList(list):
+    """
+    Like a QuerySet, but only some methods.
+    Attributes:
+    ----------
+        _root: the root json object from which the entire queryset is derived
+    """
+
+    def __init__(self, *args):
+        super().__init__(*args)
+        self._root = None
+
+    def first(self):
+        result = self.__getitem__(0)
+        return result if self.__len__() > 0 else None
+
+    def last(self, native_types_=None):
+        result = self.__getitem__(-1)
+        return result if self.__len__() > 0 else None
+
+    def exists(self):
+        return True if self.__len__() > 0 else False
+
+    def count(self):
+        return self.__len__()
+
+    def unique(self):
+        """
+        Returns unique values
+        """
+
+        # TODO add test for this
+        # TODO dict option for counting unique values
+        unique_values = ValuesList()
+        unique_values._root = self._root
+        for idx, item in enumerate(self):
+            if item not in unique_values:
+                unique_values.append(item)
+        return unique_values
+
+
 class QuerySet(list):
     """
     This is a queryset object.
@@ -197,25 +239,20 @@ class QuerySet(list):
     def __init__(self, *args):
         super().__init__(*args)
         self._root = None
+        self._native_types = None
+
+    def first(self):
 
-    def first(self, native_types_=None):
-        # ---- DYNAMIC CONFIG ----
-        if native_types_ is None:
-            native_types_ = config.native_types
-        # ------------------------
         result = self.__getitem__(0)
-        if not native_types_:
+        if not self._native_types:
             return result if self.__len__() > 0 else None
         else:  # if native_types_ is selected
             return result._data if self.__len__() > 0 else None
 
-    def last(self, native_types_=None):
-        # ---- DYNAMIC CONFIG ----
-        if native_types_ is None:
-            native_types_ = config.native_types
-        # ------------------------
+    def last(self):
+
         result = self.__getitem__(-1)
-        if not native_types_:
+        if not self._native_types:
             return result if self.__len__() > 0 else None
         else:  # if native_types_ is selected
             return result._data if self.__len__() > 0 else None
@@ -244,29 +281,85 @@ class QuerySet(list):
             else:
                 exec(f"self._root{path} = new_obj")
 
-    def unique(self):
+    def unique(self, transform=None):
         """
         Returns unique values in a querylist
+        Arguments:
+        ---------
+            transform: if selected, then applies such a function on each item in the queryset before checking
         """
 
         # TODO add test for this
         # TODO dict option for counting unique values
         unique_values = QuerySet()
         unique_values._root = self._root
-        for idx, item in enumerate(self):
-            if item not in unique_values:
-                unique_values.append(item)
-        return unique_values
+
+        transformed_values = set()
+
+        if transform is None:
+            for item in self:
+                if item not in unique_values:
+                    unique_values.append(item)
+            return unique_values
+        else:
+            for item in self:
+                obj = item
+                try:
+                    obj = transform(item)
+                except Exception:
+                    pass
+                if obj not in transformed_values:
+                    transformed_values.add(obj)
+                    unique_values.append(item)
+            return unique_values
 
     def filter(self, **q):
         # TODO add test
         output = QuerySet()
         output._root = self._root
         for item in self:
-            if item.query(**q).exists():
+            if item.parent.query(**q).exists():
                 output.append(item)
         return output
 
+    def values(self, *keys, search_upwards=True):
+        """
+        This method changes the values returned by the queryset.
+        The returned values will be python types.
+
+        Arguments
+        ---------
+            search_upwards: The function will try to search for the specified keys among the parents of each object
+
+        """
+        # TODO call node values method for each item
+
+        values_list = ValuesList(
+            (item.values(*keys, search_upwards=True) for item in self)
+        )
+        values_list._root = self._root
+        return values_list
+
+    def order_by(self, key):
+        # TODO
+        rever = True if key.startswith("-") else False
+
+        try:
+            result = QuerySet(
+                sorted(
+                    self,
+                    key=lambda x: x.parent._get(key.replace("-", "").strip(), ""),
+                    reverse=rever,
+                )
+            )
+        except Exception as e:
+            warnings.warn(f"QuerySet could not be sorted. Info: {e}")
+            # print(e)
+            return self
+
+        result._root = self._root
+        return result
+
     # ---- GROUP OPERATIONS ----
     def sum(self):
         """Sum numbers on queryset"""
@@ -315,3 +408,31 @@ class All(metaclass=AllChoices):
     """
 
     pass
+
+
+class ExtractYear:
+    # TODO add more actions
+    def __init__(self, data):
+        self.data = parsers.parse_datetime(data, fail_silently=True)
+        self.year = self.data.year if self.data else None
+
+    def exact_action(self, query_value):
+        if query_value == All:
+            return True
+
+        if not isinstance(query_value, int):
+            raise JSONQueryException(
+                f"Argument query_value must be an int, not {type(query_value)}"
+            )
+
+        return self.year == query_value
+
+    def contains_action(self, query_value):
+        if not isinstance(query_value, int):
+            raise JSONQueryException(
+                f"Argument query_value must be an int, not {type(query_value)}"
+            )
+        if self.data:
+            return str(query_value) in str(self.year)
+        else:
+            return False
