@@ -802,10 +802,125 @@ class JsonTest(unittest.TestCase):
             "root/root_list/1/child/A/",
         )
         self.assertIsNone(
-            test.get(A__parents__0=0, A__parents__index=0, throw_exceptions_=False)
+            test.get(
+                A__parents__0=0,
+                A__parents__index=0,
+                throw_exceptions_=False,
+                native_types_=True,
+            )
         )
         self.assertRaisesRegex(
             JSONQueryException,
             "Lookup parents can only be included once",
             lambda: test.get(A__parents__parents__0=0),
         )
+
+    def test_values(self):
+
+        js.config.query_exceptions = False
+
+        test = JSONObject(
+            {
+                "id": 1234,
+                "company": {
+                    "name": "NAME",
+                    "filing_set": [
+                        {
+                            "date": "2021-01-01",
+                            "data": {"field1": 1, "field2": 2},
+                            "type": "C",
+                        },
+                        {
+                            "date": "2022-01-01",
+                            "data": {"field1": 3, "field2": 4},
+                            "type": "A",
+                        },
+                    ],
+                },
+                "date": "2022-02-02",
+                "filing": {"field1": 5},
+            }
+        )
+        test2 = JSONObject(
+            {
+                "data": {
+                    "id": "0008523621",
+                    "amount": 55856212.25,
+                    "success": False,
+                    "team": [
+                        {
+                            "name": "David G. García",
+                            "age": 36,
+                            "hobbies": ["music", "reading"],
+                        },
+                        {
+                            "name": "Alexander D. Diego",
+                            "age": 31,
+                            "hobbies": ["cooking", "music", "sports", "yoga"],
+                        },
+                        {"name": "David A. Márquez", "age": 28, "hobbies": None},
+                        {
+                            "name": "Peter C. Jackson",
+                            "age": 31,
+                            "hobbies": ["sports", "reading", "tennis", "smoke"],
+                            "staff": True,
+                        },
+                    ],
+                },
+                "updated_at": "2021-05-01T09:30:00",
+                "comments": {
+                    "data": "structured",
+                    "amount": 2,
+                    "comments": [
+                        0,
+                        {
+                            "name": "Peter K. Márquez",
+                            "text": "Great!",
+                            "valuation": 10,
+                            "timestamp": "2020-03-01",
+                        },
+                        1,
+                        {
+                            "name": "Jacob H. Stack",
+                            "text": "Bad!",
+                            "valuation": 1,
+                            "timestamp": "2021-03-05",
+                            "staff": True,
+                        },
+                    ],
+                },
+            }
+        )
+
+        self.assertEqual(
+            test.get(field1=1, field1__parents__c_type="C").values("date"),
+            {"date": "2021-01-01"},
+        )
+        self.assertEqual(
+            test.get(field1=1, field1__parents__c_type="C").values("date", "name"),
+            {"date": "2021-01-01", "name": "NAME"},
+        )
+        self.assertEqual(
+            test.get(
+                field1__parents__c_date__year=2022, field1__notpath="filing"
+            ).values("date", "type", "id"),
+            {"date": "2021-01-01", "type": "C", "id": 1234},
+        )
+        self.assertEqual(
+            test.get(field1__parents__c_date__year=2022, field1__notpath="filing")
+            .values("date", "type", "id")
+            .id,
+            1234,
+        )
+
+        self.assertEqual(
+            test2.get(staff=True).values("timestamp", "age"),
+            {"timestamp": None, "age": 31},
+        )
+        self.assertEqual(
+            test2.get(fake=True).values("timestamp", "age"),
+            {"timestamp": None, "age": None},
+        )
+        self.assertIsNone(
+            test2.get(fake=True).values("timestamp", "age").timestamp,
+        )
